#include "AIGroundNetworkWriter.h"

#include "AINetworkArc.h"
#include "AINetworkNode.h"
#include "StartupLocationMapObject.h"

#include <Map/MapObjectContainer.h>

#ifdef _MSC_VER
#   define snprintf _snprintf
#endif

AIGroundNetworkWriter::AIGroundNetworkWriter(Airport::SmartRef airport):
        _airport(airport)
{
}

AIGroundNetworkWriter::~AIGroundNetworkWriter() {
}

void AIGroundNetworkWriter::write(std::ostream& outfile) {
        _vertexIndexMap.clear();
        _nextIndex=0;

        MapObjectContainer::SmartRef groundNetworkLayer=_airport->GetGroundNetworkLayer();
        MapObjectList objects;
        MapObjectList::Node *objNode;

        groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(StartupLocationMapObject));
        for (objNode=objects.GetFirst();objNode;objNode=objNode->GetNext()) {
                StartupLocationMapObject::SmartRef obj=(StartupLocationMapObject*)objNode->GetData();
                unsigned int index=_nextIndex++;
                _vertexIndexMap[obj]=index;
                //std::cerr << "Setting index for parking" << index << std::endl;
        }
        objects.Clear();
        groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(AINetworkNode));
        for (objNode=objects.GetFirst();objNode;objNode=objNode->GetNext()) {
                AINetworkNode::SmartRef obj=(AINetworkNode*)objNode->GetData();
                unsigned int index=_nextIndex++;
                _vertexIndexMap[obj]=index;
                //std::cerr << "Setting index for node" << index << std::endl;
        }


        _nextIndex = 0;
        outfile << "<?xml version=\"1.0\"?>\n";
        outfile << "<groundnet>\n";
        writeFrequencies(outfile);
        writeGates(outfile);
        writeNodes(outfile);
        writeArcs(outfile);
        outfile << "</groundnet>\n";
}

void AIGroundNetworkWriter::writeFrequencies(std::ostream& outfile) {
       outfile << "  <frequencies>\n";
       //MapObjectContainer::SmartRef info=_airport->GetObjectsLayer();
        const FrequencyEntryArray& frequencies=_airport->GetFrequencies();
        for (size_t i=0;i<frequencies.GetCount();i++) {
            FrequencyEntry freq = frequencies.Item(i);
            switch (freq.GetType()) {
		case FrequencyEntry::FrequencyType_AWOS:
			outfile << "    <AWOS>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</AWOS>" <<
                        std::endl;
			break;
		case FrequencyEntry::FrequencyType_UNICOM:
			outfile << "    <UNICOM>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</UNICOM>" << 
                        std::endl;
			break;
		case FrequencyEntry::FrequencyType_Clearance:
			outfile << "    <CLEARANCE>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</CLEARANCE>"
                         << std::endl;
			break;
		case FrequencyEntry::FrequencyType_Ground:
			outfile << "    <GROUND>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</GROUND>" << 
                        std::endl;
			break;
		case FrequencyEntry::FrequencyType_Tower:
			outfile << "    <TOWER>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</TOWER>" << 
                        std::endl;
			break;
		case FrequencyEntry::FrequencyType_Approach:
			outfile << "    <APPROACH>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</APPROACH>" 
                                << std::endl;
			break;
		case FrequencyEntry::FrequencyType_Departure:
			outfile << "    <DEPARTURE>" << freq.GetFrequency().mb_str(wxConvUTF8) << "</DEPARTURE>"
                        << std::endl;
			break;
		}
        }
       outfile << "  </frequencies>\n";
}

void AIGroundNetworkWriter::writeGates(std::ostream& outfile) {
        MapObjectContainer::SmartRef groundNetworkLayer=_airport->GetGroundNetworkLayer();
        MapObjectList objects;
        MapObjectList::Node *objNode;
        
        outfile << "  <parkingList>\n";
        
        groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(StartupLocationMapObject));
        
        for (objNode=objects.GetFirst();objNode;objNode=objNode->GetNext()) {
                StartupLocationMapObject::SmartRef obj=(StartupLocationMapObject*)objNode->GetData();
                unsigned int index=_nextIndex++;
                
                std::string gatetype;
                
                switch (obj->GetGateType()) {
                case StartupLocationMapObject::GateType_Gate:
                        gatetype=std::string("gate");
                        break;
                case StartupLocationMapObject::GateType_Cargo:
                        gatetype=std::string("cargo");
                        break;
                case StartupLocationMapObject::GateType_GA:
                        gatetype=std::string("ga");
                        break;
                case StartupLocationMapObject::GateType_Mil_Fighter:
                        gatetype=std::string("mil-fighter");
                        break;
                case StartupLocationMapObject::GateType_Mil_Cargo:
                        gatetype=std::string("mil-cargo");
                        break;
                }
               //AINetworkVertex::SmartRef ObjRef = (AINetworkVertex*) obj->getFirstPushBackPoint();
               //AINetworkVertex::SmartRef obj=(AINetworkArc*)ObjRef->GetData();
               MapObjectList::Node *refNode = (wxMapObjectListNode*) obj->getFirstPushBackPoint();
               AINetworkVertex::SmartRef ref;
               if (refNode) {
                    ref = (AINetworkNode*) refNode->GetData();
               } else {
                    ref = 0;
               }
 
                outfile << "    <Parking index=\"" << index << "\"\n"
                        << "             type=\"" << gatetype << "\"\n"
                        << "             name=\"" << (const char*)obj->GetGateName().mb_str(wxConvUTF8) << "\"\n"
                        << "             number=\"" << (const char*)obj->GetGateNumber().mb_str(wxConvUTF8) << "\"\n"
                        << "             lat=\"" << latString(obj->GetPosition().lat()) << "\"\n"
                        << "             lon=\"" << lonString(obj->GetPosition().lon()) << "\"\n"
                        << "             heading=\"" << obj->GetHeading() << "\"\n"
                        << "             radius=\"" << obj->GetRadius() << "\"\n";
                if (ref) {
                        outfile << "             pushBackRoute=\"" << _vertexIndexMap[ref] <<"\" \n";
                }
                outfile << "             airlineCodes=\"" << (const char*)obj->GetAirlineCodes().mb_str(wxConvUTF8) << "\" />\n"
                        << "\n";
        }
        
        outfile << "  </parkingList>\n";
}

void AIGroundNetworkWriter::writeNodes(std::ostream& outfile) {
        MapObjectContainer::SmartRef groundNetworkLayer=_airport->GetGroundNetworkLayer();
        MapObjectList objects;
        MapObjectList::Node *objNode;
        
        outfile << "  <TaxiNodes>\n";
        
        groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(AINetworkNode));
        
        for (objNode=objects.GetFirst();objNode;objNode=objNode->GetNext()) {
                AINetworkNode::SmartRef obj=(AINetworkNode*)objNode->GetData();
                unsigned int index=_nextIndex++;
                std::string holdPointType;
                
                switch (obj->GetHoldPointType()) {
                case AINetworkNode::HoldPointType_None:
                        holdPointType=std::string("none");
                        break;
                case AINetworkNode::HoldPointType_Normal:
                        holdPointType=std::string("normal");
                        break;
                case AINetworkNode::HoldPointType_CAT_II_III:
                        holdPointType=std::string("CAT II/III");
                        break;
                case AINetworkNode::HoldPointType_PushBack:
                        holdPointType=std::string("PushBack");
                        break;

                }
                
                //_vertexIndexMap[obj]=index;
                outfile << "    <node index=\"" << index << "\""
                        <<          " lat=\"" << latString(obj->GetPosition().lat()) << "\""
                        <<          " lon=\"" << lonString(obj->GetPosition().lon()) << "\""
                        <<          " isOnRunway=\"" << (int)obj->GetIsOnRunway() << "\""
                        <<          " holdPointType=\"" << holdPointType << "\" />\n";
        }
        
        outfile << "  </TaxiNodes>\n";
}

void AIGroundNetworkWriter::writeArcs(std::ostream& outfile) {
        MapObjectContainer::SmartRef groundNetworkLayer=_airport->GetGroundNetworkLayer();
        MapObjectList objects;
        MapObjectList::Node *objNode;
        
        outfile << "  <TaxiWaySegments>\n";
        
        groundNetworkLayer->GetObjectsOfClass(objects, CLASSINFO(AINetworkArc));
        
        for (objNode=objects.GetFirst();objNode;objNode=objNode->GetNext()) {
                AINetworkArc::SmartRef obj=(AINetworkArc*)objNode->GetData();
                AINetworkVertex::SmartRef source,target;
                
                source=obj->GetSource();
                target=obj->GetTarget();
                outfile << "    <arc begin=\"" << _vertexIndexMap[source] << "\""
                        <<         " end=\"" << _vertexIndexMap[target] << "\""
                        <<         " isPushBackRoute=\"" << (int)obj->IsPushBackRoute() << "\""
                        <<         " name=\"" << (const char*)obj->GetName().mb_str(wxConvUTF8) << "\" />\n";
                if (obj->IsBidirectional()) {
                        outfile << "    <arc begin=\"" << _vertexIndexMap[target] << "\""
                                <<         " end=\"" << _vertexIndexMap[source] << "\""
                                <<         " isPushBackRoute=\"" << (int)obj->IsPushBackRoute() << "\""
                                <<         " name=\"" << (const char*)obj->GetName().mb_str(wxConvUTF8) << "\" />\n";
                }
        }
        
        outfile << "  </TaxiWaySegments>\n";
}

std::string AIGroundNetworkWriter::latString(double _lat) {
  char result[64];
  //char hemisphere[2];
  double latPos = fabs(_lat);
  int   degree = (int) floor(latPos);
  float minute = (latPos-degree) * 60;
  if (_lat > 0)
    snprintf(result, 64, "N%02d %02.3f", degree, minute);
  else
    snprintf(result, 64, "S%02d %02.3f", degree, minute);
  
  return std::string(result);
}

std::string AIGroundNetworkWriter::lonString(double _lon) {
 char result[64];
 //char hemisphere[2];
  double lonPos = fabs(_lon);
  int   degree = (int) floor(lonPos);
  float minute = (lonPos-degree) * 60;
  if (_lon > 0)
    snprintf(result, 64, "E%02d %02.3f", degree, minute);
  else
    snprintf(result, 64, "W%02d %02.3f", degree, minute);
  return std::string(result);
}

