#include "AINetworkActionHandler.h"

#include "AINetworkArc.h"
#include "AINetworkArcSelection.h"
#include "AINetworkVertex.h"
#include "AINetworkVertexSelection.h"

#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(AINetworkActionHandler,wxEvtHandler);

AINetworkActionHandler AINetworkActionHandler::_instance;

AINetworkActionHandler::AINetworkActionHandler():
        wxEvtHandler()
{
}

AINetworkActionHandler::~AINetworkActionHandler() {
}

void AINetworkActionHandler::DoConnectNodes(MapEditor* editor, bool bidirectional) {
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList selections=selmgr->GetSelections();
        MapSelectionList::Node* node=selections.GetFirst();
        wxASSERT(node!=NULL);
                                                               
        AINetworkVertexSelection* lastVertexSelection=(AINetworkVertexSelection*)node->GetData();
        AINetworkVertex::SmartRef lastVertex=lastVertexSelection->GetContent();
        for (node=node->GetNext();node;node=node->GetNext()) {
                AINetworkVertexSelection* vertexSelection=(AINetworkVertexSelection*)node->GetData();
                AINetworkVertex::SmartRef vertex=vertexSelection->GetContent();
                AINetworkArc::SmartRef arc=new AINetworkArc(lastVertex->GetTransactionManager(),
                        lastVertex,vertex,
                        bidirectional,
                        false);
                lastVertex->GetContainer()->AttachObject(arc);
                lastVertex=vertex;
        }
}

void AINetworkActionHandler::OnConnectNodesUnidirectional(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        DoConnectNodes(editor,false);
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Connecting AI nodes unidirectionally"));
}

void AINetworkActionHandler::OnConnectNodesBidirectional(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        DoConnectNodes(editor,true);
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Connecting AI nodes bidirectionally"));
}

void AINetworkActionHandler::OnMergeNodesAtFirst(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList selections=selmgr->GetSelections();
        MapSelectionList::Node* node=selections.GetFirst();
        wxASSERT(node!=NULL);
        
        AINetworkVertexSelection* firstVertexSelection=(AINetworkVertexSelection*)node->GetData();
        AINetworkVertex::SmartRef firstVertex=firstVertexSelection->GetContent();
        for (node=node->GetNext();node;node=node->GetNext()) {
                AINetworkVertexSelection* vertexSelection=(AINetworkVertexSelection*)node->GetData();
                AINetworkVertex::SmartRef vertex=vertexSelection->GetContent();
                firstVertex->MergeArcsFrom(vertex);
                vertex->RemoveFromContainer();
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Merging AI nodes"));
}

void AINetworkActionHandler::OnMergeNodesAtLast(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList selections=selmgr->GetSelections();
        MapSelectionList::Node* node=selections.GetLast();
        wxASSERT(node!=NULL);
        
        AINetworkVertexSelection* firstVertexSelection=(AINetworkVertexSelection*)node->GetData();
        AINetworkVertex::SmartRef firstVertex=firstVertexSelection->GetContent();
        for (node=node->GetPrevious();node;node=node->GetPrevious()) {
                AINetworkVertexSelection* vertexSelection=(AINetworkVertexSelection*)node->GetData();
                AINetworkVertex::SmartRef vertex=vertexSelection->GetContent();
                firstVertex->MergeArcsFrom(vertex);
                vertex->RemoveFromContainer();
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Merging AI nodes"));
}

void AINetworkActionHandler::OnFlipUnidirectionalEdges(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList::Node* node;

        for (node=selmgr->GetSelections().GetFirst();node;node=node->GetNext()) {
                AINetworkArcSelection* arcSelection=(AINetworkArcSelection*)node->GetData();
                AINetworkArc::SmartRef arc=arcSelection->GetContent();
                AINetworkVertex::SmartRef source,target;
                source=arc->GetSource();
                target=arc->GetTarget();
                
                arc->SetSource(target);
                arc->SetTarget(source);
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Flipping AI arcs"));
}

void AINetworkActionHandler::OnSplitAtCenter(wxCommandEvent& evt) {
        wxMenu *menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapSelectionManager::SmartRef selmgr=&editor->GetSelectionManager();
        MapSelectionList selections=selmgr->GetSelections();
        MapSelectionList::Node* node;

        for (node=selections.GetFirst();node;node=node->GetNext()) {
                AINetworkArcSelection* arcSelection=(AINetworkArcSelection*)node->GetData();
                AINetworkArc::SmartRef arc=arcSelection->GetContent();
                AINetworkArc::SmartRef newarc=arc->SplitAtCenter(editor);
                selmgr->AddSelectionFor(newarc);
        }
        TransactionManager* tmgr=editor->GetTransactionManager();
        tmgr->Commit(wxT("Splitting AI arcs"));
}

