#include "AINetworkNode.h"

#include "AINetworkNodeSelection.h"

#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Math/CollisionUtils.h>

IMPLEMENT_CLASS(AINetworkNode,AINetworkVertex)

double AINetworkNode::Radius=5.0;

AINetworkNode::AINetworkNode(TransactionManager* transactionManager,
                             Point3D position,
                             bool isOnRunway,
                             HoldPointType holdPointType):
        AINetworkVertex(transactionManager,position)
{
        TRANSACTIONOBJECT_STATEVAR(isOnRunway)=isOnRunway;
        TRANSACTIONOBJECT_STATEVAR(holdPointType)=holdPointType;
}

AINetworkNode::~AINetworkNode() {
}

void AINetworkNode::Paint(wxDC& dc, MapEditor* editor, const MapPass& pass) {
        if (pass!=AINetworkPass)
                return;
        PaintNode(dc,editor,wxColour(192,192,0));
}

void AINetworkNode::PaintNode(wxDC& dc, MapEditor* editor, wxColour colour) {
        MapCoordinateSystem* cs=&editor->GetCoordinateSystem();
        Point3D localPos=editor->GetProjection()->ConvertToLocal(GetPosition());
        wxPoint center=cs->LogicalToDevice(localPos);
        
        wxCoord pointSize=cs->BoundedDeviceSizeX(3,15,Radius);
        
        wxPen pen(colour,1,wxSOLID);
        wxBrush brush(colour,wxSOLID);
        
        /* Draw ring */
        dc.SetPen(pen);
        dc.SetBrush(brush);
        dc.DrawCircle(center,pointSize);
        dc.SetPen(wxNullPen);
        dc.SetBrush(wxNullBrush);
}

bool AINetworkNode::Hit(MapEditor* editor, TDRectangle r) {
        MapCoordinateSystem* cs=&editor->GetCoordinateSystem();
        Point3D localPos=editor->GetProjection()->ConvertToLocal(GetPosition());
        double pointSize=cs->BoundedLogicalSizeX(3,15,Radius);;
        return CollisionUtils::IntersectsCircle(r,localPos,pointSize);
}

MapSelection* AINetworkNode::CreateSelection(MapSelectionManager* manager) {
        return new AINetworkNodeSelection(manager, this);
}

wxRect AINetworkNode::GetDeviceBounds(MapEditor* editor) {
        MapCoordinateSystem* cs=&editor->GetCoordinateSystem();
        Point3D localPos=editor->GetProjection()->ConvertToLocal(GetPosition());
        wxPoint center=cs->LogicalToDevice(localPos);

        wxCoord pointSize=cs->BoundedDeviceSizeX(3,15,Radius);
        
        return wxRect(center.x-pointSize,center.y-pointSize,
                      2*pointSize+1,2*pointSize+1);
}

TDRectangle AINetworkNode::GetGeographicBounds() {
        return TDRectangle(GetPosition());
}

