#include "AINetworkVertexSelection.h"
#include "AINetworkVertex.h"
#include "AINetworkActionHandler.h"
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(AINetworkVertexSelection,MapSelection);

AINetworkVertexSelection::AINetworkVertexSelection(MapSelectionManager* manager, AINetworkVertex* content):
        MapSelection(manager,content) {
}

AINetworkVertexSelection::~AINetworkVertexSelection() {
}

void AINetworkVertexSelection::ProvideActionMenuForClass(wxMenu& popupMenu, wxClassInfo* targetClass) {
        MapSelection::ProvideActionMenuForClass(popupMenu,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkVertexSelection)))
                return;
        wxMenu* vertexMenu=new wxMenu();
        vertexMenu->SetEventHandler(popupMenu.GetEventHandler());
        vertexMenu->SetClientData(popupMenu.GetClientData());
        wxMenuItem *item;
        
        item=vertexMenu->Append(wxID_ANY,wxT("Connect unidirectionally"));
        vertexMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(AINetworkActionHandler::OnConnectNodesUnidirectional),
                                              NULL,
                                              AINetworkActionHandler::GetInstance());
        item=vertexMenu->Append(wxID_ANY,wxT("Connect bidirectionally"));
        vertexMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(AINetworkActionHandler::OnConnectNodesBidirectional),
                                              NULL,
                                              AINetworkActionHandler::GetInstance());
        item=vertexMenu->Append(wxID_ANY,wxT("Merge at first"));
        vertexMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(AINetworkActionHandler::OnMergeNodesAtFirst),
                                              NULL,
                                              AINetworkActionHandler::GetInstance());
        item=vertexMenu->Append(wxID_ANY,wxT("Merge at last"));
        vertexMenu->GetEventHandler()->Connect(item->GetId(),
                                              wxEVT_COMMAND_MENU_SELECTED,
                                              wxCommandEventHandler(AINetworkActionHandler::OnMergeNodesAtLast),
                                              NULL,
                                              AINetworkActionHandler::GetInstance());
        popupMenu.Append(wxID_HIGHEST,wxT("AI Nodes"),vertexMenu);
}

void AINetworkVertexSelection::DeltaDrag(Point3D delta) {
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapProjection* proj=editor->GetProjection();
        
        Point3D pos=GetContent()->GetPosition();
        Point3D localPos=proj->ConvertToLocal(pos);
        localPos+=delta;
        pos=proj->ConvertFromLocal(localPos);
        
        GetContent()->SetPosition(pos);
}

void AINetworkVertexSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkVertexSelection)))
                return;
        
        propgrid->Append(wxFloatProperty(wxT("Latitude (deg)"),wxT("AINetworkVertexSelection_Lat")));
        propgrid->Append(wxFloatProperty(wxT("Longitude (deg)"),wxT("AINetworkVertexSelection_Lon")));
}

void AINetworkVertexSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        MapSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkVertexSelection)))
                return;

        Point3D center=GetContent()->GetPosition();
        propgrid->SetPropertyValue(wxT("AINetworkVertexSelection_Lat"),center.lat());
        propgrid->SetPropertyValue(wxT("AINetworkVertexSelection_Lon"),center.lon());
}

void AINetworkVertexSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        MapSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(AINetworkVertexSelection)))
                return;
        AINetworkVertex* firstContent=wxDynamicCast(firstSelection,AINetworkVertexSelection)->GetContent();
        AINetworkVertex* myContent=GetContent();

        if (myContent->GetPosition().lat()!=firstContent->GetPosition().lat())
                propgrid->SetPropertyUnspecified(wxT("AINetworkVertexSelection_Lat"));
        if (myContent->GetPosition().lon()!=firstContent->GetPosition().lon())
                propgrid->SetPropertyUnspecified(wxT("AINetworkVertexSelection_Lon"));
}

void AINetworkVertexSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("AINetworkVertexSelection_Lat"))==0) {
                Point3D center=GetContent()->GetPosition();
                center.setlat(value.GetDouble());
                GetContent()->SetPosition(center);
        } else if (name.Cmp(wxT("AINetworkVertexSelection_Lon"))==0) {
                Point3D center=GetContent()->GetPosition();
                center.setlon(value.GetDouble());
                GetContent()->SetPosition(center);
        } else {
                MapSelection::UpdateObjectFromProperty(property);
        }
}

