#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "BeaconSelection.h"
#include "BeaconMapObject.h"
#include <Map/MapEditor.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(BeaconSelection,PointObjectSelection);

BeaconSelection::BeaconSelection(MapSelectionManager* manager, BeaconMapObject* content):
        PointObjectSelection(manager,content) {
}

BeaconSelection::~BeaconSelection() {
}

void BeaconSelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PointObjectSelection::ProvidePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(BeaconSelection)))
                return;
        
        wxPGChoices beaconTypeChoices;
        beaconTypeChoices.Add(wxT("None"),BeaconMapObject::Beacon_None);
        beaconTypeChoices.Add(wxT("white-green flash (land airport)"),BeaconMapObject::Beacon_WhiteGreen_Flash);
        beaconTypeChoices.Add(wxT("white-yellow flash (seaplane base)"),BeaconMapObject::Beacon_WhiteYellow_Flash);
        beaconTypeChoices.Add(wxT("green-yellow-white flash (heliports)"),BeaconMapObject::Beacon_GreenYellowWhite_Flash);
        beaconTypeChoices.Add(wxT("white-white-green flashi (military field)"),BeaconMapObject::Beacon_WhiteWhiteGreen_Flash);
        beaconTypeChoices.Add(wxT("white strobe"),BeaconMapObject::Beacon_White_Strobe);

        propgrid->Append(wxEnumProperty(wxT("Type"),wxT("BeaconSelection_Type"),beaconTypeChoices));
        
        propgrid->Append(wxStringProperty(wxT("Name"), wxT("BeaconSelection_Name")));
}

void BeaconSelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PointObjectSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(BeaconSelection)))
                return;

        propgrid->SetPropertyValue(wxT("BeaconSelection_Type"),GetContent()->GetBeaconType());
        propgrid->SetPropertyValue(wxT("BeaconSelection_Name"),GetContent()->GetName());
}

void BeaconSelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        PointObjectSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(BeaconSelection)))
                return;
        BeaconMapObject* firstContent=wxDynamicCast(firstSelection,BeaconSelection)->GetContent();
        BeaconMapObject* myContent=GetContent();

        if (firstContent->GetBeaconType()!=myContent->GetBeaconType())
                propgrid->SetPropertyUnspecified(wxT("BeaconSelection_Type"));
        if (firstContent->GetName().Cmp(myContent->GetName())!=0)
                propgrid->SetPropertyUnspecified(wxT("BeaconSelection_Name"));
}

void BeaconSelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        wxVariant value=property->GetValueAsVariant();

        if (name.Cmp(wxT("BeaconSelection_Type"))==0) {
                GetContent()->SetBeaconType((BeaconMapObject::BeaconType)value.GetLong());
        } else if (name.Cmp(wxT("BeaconSelection_Name"))==0) {
                GetContent()->SetName(property->GetValueAsString());
        } else {
                PointObjectSelection::UpdateObjectFromProperty(property);
        }
}


