#ifndef PointMapObject_h
#define PointMapObject_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "Airport.h"

#include <Map/MapObject.h>

class PointMapObject: public MapObject {
        
        DECLARE_CLASS(PointMapObject)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(PointMapObject);
        
        friend class PointObjectSelection;
public:
        PointMapObject(TransactionManager* transactionManager,
                       Point3D position,
                       wxPen outerRingPen,
                       wxPen innerRingPen,
                       wxBrush innerRingBrush);
        
        /* May only be called by MapLayer */
        virtual ~PointMapObject();
        
        /* Paint this object using the given device context into the given editor.
         * Pass is the pass number, starting with 0 for the first pass. */
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        /* Return TRUE iff this object intersects the given hit-TDRectangle r to list.
         * r is given in the local coordinate system of editor. */
        virtual bool Hit(MapEditor* editor, TDRectangle r);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        /* Return the geographic (lat/lon) bounds */
        virtual TDRectangle GetGeographicBounds();
        
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR_WCOMP(Point3D,Position,_position)
protected:
        typedef struct {
                Point3D _position;
        } State;
        
        TRANSACTIONOBJECT_DEFINE_STATE_MEMENTO(PointMapObject,MapObject,State)
protected:
        const wxPen _outerRingPen;
        const wxPen _innerRingPen;
        const wxBrush _innerRingBrush;
};

#endif

