#include "RunwaySelectionLengthHandle.h"

#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelectionManager.h>

IMPLEMENT_CLASS(RunwaySelectionLengthHandle,PavementSelectionSideHandle);

RunwaySelectionLengthHandle::RunwaySelectionLengthHandle(RunwaySelection* selection, int end, EndType endtype):
        PavementSelectionSideHandle(selection,2*end),
        _end(end),
        _endtype(endtype)
{
        RunwayMapObject* object=selection->GetContent();

        if (_end==0) {
                _thisend =object->GetDesignatedEnd();
                _otherend=object->GetOppositeEnd();
        } else {
                _thisend =object->GetOppositeEnd();
                _otherend=object->GetDesignatedEnd();
        }
        
        _swyendpos2=(double)_otherend.stopwayLength;
        _threshpos2=_swyendpos2+(double)_otherend.displacedThreshold;
        _swyendpos1=_swyendpos2+object->GetLength();
        _threshpos1=_swyendpos1-(double)_thisend.displacedThreshold;
        _endpos1   =_swyendpos1+(double)_thisend.stopwayLength;
}

RunwaySelectionLengthHandle::~RunwaySelectionLengthHandle()
{
}

void RunwaySelectionLengthHandle::StartDrag(Point3D startPosition) {
        PavementSelectionSideHandle::StartDrag(startPosition,
                                               GetCorners(),
                                               2*_end);
}

void RunwaySelectionLengthHandle::Drag(Point3D currentPosition) {
        RunwaySelection* selection=GetSelection();
        RunwayMapObject* object=selection->GetContent();
        MapEditor* editor=selection->GetSelectionManager()->GetEditor();
        MapProjection* projection=editor->GetProjection();
        
        Point3D localcenters[2];
        double length;
        
        length=GetNewLength(currentPosition,
                            GetCorners(),
                            2*_end,
                            localcenters);
        
        
        double new_swyendpos2,new_threshpos2,new_threshpos1,new_swyendpos1,new_endpos1;
        RunwayMapObject::RunwayEndData current_thisend,current_otherend;

        new_swyendpos2=_swyendpos2;
        new_threshpos2=_threshpos2;
        new_threshpos1=_threshpos1;
        new_swyendpos1=_swyendpos1;
        new_endpos1   =_endpos1;

        if (_end==0) {
                current_thisend =object->GetDesignatedEnd();
                current_otherend=object->GetOppositeEnd();
        } else {
                current_thisend =object->GetOppositeEnd();
                current_otherend=object->GetDesignatedEnd();
        }
        
        switch (_endtype) {
        case EndType_RunwayEnd:
                new_endpos1=length;
                if (new_endpos1<0) {
                        new_endpos1=0;
                }
                if (new_endpos1<new_swyendpos1) {
                        new_swyendpos1=new_endpos1;
                }
                if (new_endpos1<new_threshpos1) {
                        new_threshpos1=new_endpos1;
                }
                if (new_endpos1<new_threshpos2) {
                        new_threshpos2=new_endpos1;
                }
                if (new_endpos1<new_swyendpos2) {
                        new_swyendpos2=new_endpos1;
                }
                break;
        case EndType_StopwayEnd:
                new_swyendpos1=length+(double)current_otherend.stopwayLength;
                if (new_swyendpos1<0) {
                        new_swyendpos1=0;
                }
                if (new_endpos1<new_swyendpos1) {
                        /*
                         * To allow the user to just collapse the stopway
                         * towards the runway end we clip the stopway position.
                         */
                        new_swyendpos1=new_endpos1;
                }
                if (new_swyendpos1<new_threshpos1) {
                        new_threshpos1=new_swyendpos1;
                }
                if (new_swyendpos1<new_threshpos2) {
                        new_threshpos2=new_swyendpos1;
                }
                if (new_swyendpos1<new_swyendpos2) {
                        new_swyendpos2=new_swyendpos1;
                }
                break;
        case EndType_Threshold:
                new_threshpos1=length+(double)(current_otherend.displacedThreshold+current_otherend.stopwayLength);
                if (new_threshpos1<0) {
                        new_threshpos1=0;
                }
                if (new_endpos1<new_threshpos1) {
                        /*
                         * To allow the user to just collapse the threshold
                         * towards the runway end we clip the stopway position.
                         */
                        new_threshpos1=new_endpos1;
                }
                if (new_swyendpos1<new_threshpos1) {
                        new_swyendpos1=new_threshpos1;
                }
                if (new_threshpos1<new_threshpos2) {
                        new_threshpos2=new_threshpos1;
                }
                if (new_threshpos1<new_swyendpos2) {
                        new_swyendpos2=new_threshpos1;
                }
                break;
        }
        
        current_otherend.stopwayLength=(int)new_swyendpos2;
        current_otherend.displacedThreshold=(int)(new_threshpos2-new_swyendpos2);
        current_thisend.displacedThreshold=(int)(new_swyendpos1-new_threshpos1);
        current_thisend.stopwayLength=(int)(new_endpos1-new_swyendpos1);
        double newLength=new_endpos1-current_thisend.stopwayLength-current_otherend.stopwayLength;
        
        /* Determine the length between localcenters[1] and the new runway center */
        double refhalflength=newLength/2.0;
        switch (_endtype) {
        case EndType_RunwayEnd:
                refhalflength+=(double)current_otherend.stopwayLength;
                break;
        case EndType_StopwayEnd:
                break;
        case EndType_Threshold:
                refhalflength-=(double)current_otherend.displacedThreshold;
                break;
        }
        
        /*
         * Determine new center
         */
        Point3D localCenter=localcenters[1]+(localcenters[0]-localcenters[1])*refhalflength/length;
        Point3D center=projection->ConvertFromLocal(localCenter);

        object->SetLength(newLength);
        object->SetCenter(center);
        if (_end==0) {
                object->SetDesignatedEnd(current_thisend);
                object->SetOppositeEnd(current_otherend);
        } else {
                object->SetDesignatedEnd(current_otherend);
                object->SetOppositeEnd(current_thisend);
        }
}

wxString RunwaySelectionLengthHandle::EndDrag() {
        // FIXME: should delete this object now
        switch (_endtype) {
        case EndType_RunwayEnd:
                return wxT("Changing total runway length");
        case EndType_StopwayEnd:
                return wxT("Changing stopway length");
        case EndType_Threshold:
                return wxT("Changing displaced threshold");
        }
}

/* Add actions to the actions menu. */
void RunwaySelectionLengthHandle::ProvideActionMenu(wxMenu& actionMenu)
{
        PavementSelectionSideHandle::ProvideActionMenu(actionMenu);
}

const Point3D* RunwaySelectionLengthHandle::GetCorners() {
        RunwaySelection* selection=GetSelection();
        RunwayMapObject* object=selection->GetContent();

        switch (_endtype) {
        case EndType_RunwayEnd:
                return object->GetEndCorners();
        case EndType_StopwayEnd:
                return object->GetStopwayCorners();
        case EndType_Threshold:
                return object->GetThresholdCorners();
        }
}

