#ifndef CalibrationMode_h
#define CalibrationMode_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Map/Mode.h>

#include "MapCalibratedBackgroundImage.h"

#include <Math/point3d.h>

#include <vector>

class CalibrationMode : public Mode {
        
        DECLARE_CLASS(CalibrationMode)
        
        typedef std::vector<Point3D> Point3DList;
public:
        CalibrationMode(MapCalibratedBackgroundImage::SmartRef bgImg,
                        bool doRotate,
                        bool hasFixedScale);
        virtual ~CalibrationMode();
        
        virtual bool Enter();
        
        virtual void Leave(bool force=false);
        
        virtual wxString GetStatus();
        
        void OnLeftDown(wxMouseEvent& evt);
        void OnLeftUp(wxMouseEvent& evt);
	void OnLeftDClick(wxMouseEvent& evt);
protected:
        void CalculateUniformScaleNoRotate();
        void CalculateFixedScaleNoRotate();
        void CalculateUniformScaleRotate();
        void CalculateFixedScaleRotate();
	double CalculateSigma(double A, double B, double C,
			      double D, double E, double F);
private:
        DECLARE_EVENT_TABLE()
        
        MapCalibratedBackgroundImage::SmartRef _bgImg;
        bool _doRotate;
        bool _hasFixedScale;
        bool _acceptClick;
        bool _expectImageClick;
        size_t _pointCount;
        double _fixedScale;
        
        double _sumx,_sumy,
                _sumu,_sumv,
                _sumux,_sumuy,
                _sumvx,_sumvy,
                _sumxx,_sumyy,
                _sumxy,
                _sumuu,_sumvv;

        Point3DList _imagePoints;
        Point3DList _mapPoints;
};

#endif


