#include "USGSLoadMode.h"

#include "USGSFetcher.h"

#include <Map/MapEditor.h>
#include <Map/MapProjection.h>

IMPLEMENT_CLASS(USGSLoadMode,Mode)

BEGIN_EVENT_TABLE(USGSLoadMode,Mode)
        EVT_LEFT_DOWN(USGSLoadMode::OnLeftButtonDown)
        EVT_MOTION(USGSLoadMode::OnMouseMotion)
        EVT_LEFT_UP(USGSLoadMode::OnLeftButtonUp)
        EVT_PAINT(USGSLoadMode::OnPaint)
END_EVENT_TABLE()

USGSLoadMode::USGSLoadMode(MapUSGSImageLayer::SmartRef targetLayer,
                           wxFileName imgPath):
        _boxStarted(false),
        _targetLayer(targetLayer),
        _imgPath(imgPath) {
}

USGSLoadMode::~USGSLoadMode() {
}

wxString USGSLoadMode::GetStatus() {
        return wxT("Drag the mouse to define the image area");
}

void USGSLoadMode::OnLeftButtonDown(wxMouseEvent& evt) {
        if (evt.AltDown() || evt.ControlDown() || evt.ShiftDown()) {
                evt.Skip();
                return;
        }
        
        _endPoint=_startPoint=evt.GetPosition();
        _boxStarted=true;
}

void USGSLoadMode::OnMouseMotion(wxMouseEvent& evt) {
        if (!_boxStarted) {
                evt.Skip();
                return;
        }
        
        wxRect oldRect=GetCurrentDeviceRectangle();
        _endPoint=evt.GetPosition();
        wxRect newRect=GetCurrentDeviceRectangle();
        GetEditor()->DamageRect(oldRect);
        GetEditor()->DamageRect(newRect);
}

void USGSLoadMode::OnLeftButtonUp(wxMouseEvent& evt) {
        if (!_boxStarted) {
                evt.Skip();
                return;
        }
        _endPoint=evt.GetPosition();
        
        /* Force redraw on the region covered by the selection rectangle */
        GetEditor()->DamageRect(GetCurrentDeviceRectangle());
        
        Point3D logicalStartPoint,logicalEndPoint;
        MapCoordinateSystem& cs=GetEditor()->GetCoordinateSystem();
        logicalStartPoint=cs.DeviceToLogical(_startPoint);
        logicalEndPoint=cs.DeviceToLogical(_endPoint);
        
        MapProjection* proj=GetEditor()->GetProjection();
        TDRectangle geoBounds(proj->ConvertFromLocal(logicalStartPoint),
                              proj->ConvertFromLocal(logicalEndPoint));
        
        
        _targetLayer->FetchImagesForRectangle(geoBounds,_imgPath);
        
        GetEditor()->GetTransactionManager()->Commit(wxT("Loading USGS images"));
        
        _boxStarted=false;
}

void USGSLoadMode::OnPaint(wxPaintEvent& evt) {
        if (!_boxStarted) {
                evt.Skip();
                return;
        }
        wxClientDC dc(GetEditor()->GetDrawingWindow());
        wxRect rect=GetCurrentDeviceRectangle();
        
        wxPen pen(*wxBLACK,1,wxDOT);
        dc.SetPen(pen);
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        dc.DrawRectangle(rect);
        dc.SetPen(wxNullPen);
        dc.SetBrush(wxNullBrush);
        
        // Pass on to other modes
        evt.Skip();
}

wxRect USGSLoadMode::GetCurrentDeviceRectangle() {
        wxCoord top,bottom,right,left;
        
        left=std::min(_startPoint.x,_endPoint.x);
        right=std::max(_startPoint.x,_endPoint.x);
        top=std::min(_startPoint.y,_endPoint.y);
        bottom=std::max(_startPoint.y,_endPoint.y);
        
        return wxRect(left,top,right-left,bottom-top);
}
