#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapFrame.h"
#include "MapViewManager.h"
#include "ProjectWizardNewAirportPage.h"

IMPLEMENT_CLASS(ProjectWizardNewAirportPage,wxWizardPageSimple);

BEGIN_EVENT_TABLE(ProjectWizardNewAirportPage,wxWizardPageSimple)
        EVT_WIZARD_PAGE_CHANGING(wxID_ANY,ProjectWizardNewAirportPage::OnPageChanging)
END_EVENT_TABLE()

ProjectWizardNewAirportPage::ProjectWizardNewAirportPage(wxWizard* parent,
                     wxWizardPage* prev,
                     wxWizardPage* next,
                     const wxBitmap& bitmap,
                     const wxChar *resource):
             wxWizardPageSimple(parent,prev,next,bitmap,resource)
{
        wxFlexGridSizer* topSizer=new wxFlexGridSizer(2,10,10);
        topSizer->SetFlexibleDirection(wxHORIZONTAL);
        topSizer->AddGrowableCol(1,1);
        
        wxStaticText *text;
        
        text=new wxStaticText(this,wxID_ANY,wxT("Aiport Type"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        wxArrayString choices;
        choices.Add(wxT("Land Airport"));
        choices.Add(wxT("Sea Base"));
        choices.Add(wxT("Heliport"));
        
        _airportType=new wxChoice(this,wxID_ANY,wxDefaultPosition,wxDefaultSize,choices);
        topSizer->Add(_airportType);
        
        text=new wxStaticText(this,wxID_ANY,wxT("ICAO ID"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        _icaoID=new wxTextCtrl(this,wxID_ANY,wxT("KSFO"));
        topSizer->Add(_icaoID);
        
        text=new wxStaticText(this,wxID_ANY,wxT("Name"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        _name=new wxTextCtrl(this,wxID_ANY,wxT("San Francisco Intl"));
        wxSize ctrlSize;
        ctrlSize=_name->GetSize();
        ctrlSize.SetWidth(200);
        _name->SetMinSize(ctrlSize);
        topSizer->Add(_name,wxSizerFlags(1).Expand());
        
        text=new wxStaticText(this,wxID_ANY,wxT("Elevation (ft)"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        _elevation=new wxTextCtrl(this,wxID_ANY,wxT("13"));
        topSizer->Add(_elevation);
        
        text=new wxStaticText(this,wxID_ANY,wxT("Has control tower"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        _hasControlTower=new wxCheckBox(this,wxID_ANY,wxEmptyString);
        topSizer->Add(_hasControlTower);
        
        text=new wxStaticText(this,wxID_ANY,wxT("Has default buildings"));
        topSizer->Add(text,wxALIGN_LEFT);
        
        _hasDefaultBuildings=new wxCheckBox(this,wxID_ANY,wxEmptyString);
        topSizer->Add(_hasDefaultBuildings);
        
        text=new wxStaticText(this,wxID_ANY,wxT("Reference Latitude"));
        topSizer->Add(text,wxSizerFlags(0).Left());
        
        _refLat=new wxTextCtrl(this,wxID_ANY,wxT("37.616"));
        topSizer->Add(_refLat,wxSizerFlags(1).Expand().Left());
        
        text=new wxStaticText(this,wxID_ANY,wxT("Reference Longitude"));
        topSizer->Add(text,wxSizerFlags(0).Left());
        
        _refLon=new wxTextCtrl(this,wxID_ANY,wxT("-122.383"));
        topSizer->Add(_refLon,wxSizerFlags(1).Expand().Left());
        
        SetSizer(topSizer);
        
        topSizer->SetSizeHints(this);
        
        Fit();
}

ProjectWizardNewAirportPage::~ProjectWizardNewAirportPage() {
}

Airport::AirportType ProjectWizardNewAirportPage::GetAirportType() const {
        switch (_airportType->GetCurrentSelection()) {
        default:
        case 0:
                return Airport::AirportType_Land;
        case 1:
                return Airport::AirportType_Sea;
        case 2:
                return Airport::AirportType_Heliport;
        }
}

void ProjectWizardNewAirportPage::OnPageChanging(wxWizardEvent& evt) {
        if (!evt.GetDirection())
                return;
        
        double elevation,lat,lon;
        wxString str;
        
        str=_elevation->GetValue();
        if (!str.ToDouble(&elevation)) {
                ::wxMessageBox(wxT("Elevation must be a number"),
                               wxT("Invalid Input"),
                               wxOK|wxICON_WARNING,
                               this);
                evt.Veto();
                return;
        }
        
        str=_refLat->GetValue();
        if (!str.ToDouble(&lat) || lat<-90.0 || lat>90.0) {
                ::wxMessageBox(wxT("Reference Latitude must be a number between -90.0 and 90.0"),
                               wxT("Invalid Input"),
                               wxOK|wxICON_WARNING,
                               this);
                evt.Veto();
                return;
        }
        
        str=_refLon->GetValue();
        if (!str.ToDouble(&lon) || lon<-180.0 || lon>180.0) {
                ::wxMessageBox(wxT("Reference Latitude must be a number between -180.0 and 180.0"),
                               wxT("Invalid Input"),
                               wxOK|wxICON_WARNING,
                               this);
                evt.Veto();
                return;
        }
        
        TransactionManager* tmgr=new TransactionManager();
        
        Airport::SmartRef airport=new Airport(tmgr,
                _icaoID->GetValue().MakeUpper(),
                _name->GetValue(),
                GetAirportType(),
                elevation,
                _hasControlTower->GetValue(),
                _hasDefaultBuildings->GetValue());
                        
        MapFrame* newFrame=new MapFrame(airport);
        MapEditor* newEditor=newFrame->GetEditor();
        
        Point3D center;
        center.setlat(lat);
        center.setlon(lon);
        
        MapProjection* proj=newEditor->GetProjection();
        proj->Init(center);
        
        newEditor->GetTransactionManager()->Clear();
        
        Point3D localCenter=proj->ConvertToLocal(center);
        TDRectangle bounds=TDRectangle(localCenter);
        bounds.Enlarge(3000,3000);
        newEditor->ZoomTo(bounds);
        
        newFrame->Show();
        MapViewManager::GetInstance()->ActivateView(newFrame);
}

