#include "MapObjectPropertyManager.h"

#include "MapEditor.h"
#include "MapSelectionManager.h"

IMPLEMENT_CLASS(MapObjectPropertyManager,MapPropertyManager);

MapObjectPropertyManager::MapObjectPropertyManager(MapEditor* editor, wxPropertyGrid* grid):
        MapPropertyManager(grid),
        _editor(editor),
        _selectionChanged(false)
{
        _editor->GetSelectionManager().RegisterSelectionChangeListener(this);
}

MapObjectPropertyManager::~MapObjectPropertyManager() {
        _editor->GetSelectionManager().UnregisterSelectionChangeListener(this);
}

void MapObjectPropertyManager::UpdateProperties() {
        MapSelectionManager& mgr=_editor->GetSelectionManager();
        
        wxClassInfo* commonSuperClass;
        commonSuperClass=mgr.FindCommonSelectionSuperClass();
        
        GetManagedGrid()->Freeze();
        
        if (_selectionChanged) {
                GetManagedGrid()->Clear();
                
                if (mgr.GetSelections().GetCount()!=0) {
                        MapSelection* representative=mgr.FindFirstSelectionOfType(commonSuperClass);
                        representative->ProvidePropertiesForClass(GetManagedGrid(),commonSuperClass);
                }
                _selectionChanged=false;
        }
        
        if (mgr.GetSelections().GetCount()!=0) {
                MapSelectionList::Node* node;
                MapSelection* firstSelection;
                node=mgr.GetSelections().GetFirst();
                firstSelection=node->GetData();
                firstSelection->UpdatePropertiesForClass(GetManagedGrid(),commonSuperClass);
                
                for (node=node->GetNext();node;node=node->GetNext()) {
                        node->GetData()->MarkDifferingPropertiesForClass(GetManagedGrid(),commonSuperClass,firstSelection);
                }
        }
        
        GetManagedGrid()->Thaw();
        
        MapPropertyManager::UpdateProperties();
}

void MapObjectPropertyManager::PropertyChanged(wxPropertyGridEvent& evt) {
        wxPGId propid=evt.GetProperty();
        wxPGProperty* prop=GetManagedGrid()->GetPropertyPtr(propid);

        MapSelectionManager& mgr=_editor->GetSelectionManager();

        MapSelectionList::Node* node;
        for (node=mgr.GetSelections().GetFirst();node;node=node->GetNext()) {
                node->GetData()->UpdateObjectFromProperty(prop);
        }
        
        TransactionManager* transMgr=_editor->GetTransactionManager();
        transMgr->Commit(wxT("Changed property ")+prop->GetLabel());
}

void MapObjectPropertyManager::SelectionsAdded(MapSelectionManager* mgr, const MapSelectionList& sellist) {
        wxASSERT_MSG(mgr==&_editor->GetSelectionManager(),
                        wxT("MapObjectPropertyManager::SelectionsAdded() called from wrong selection manager"));
        MapSelectionList::Node* node;
        for (node=sellist.GetFirst();node;node=node->GetNext())
                node->GetData()->GetContent()->RegisterChangeListener(this);
        _selectionChanged=true;
        SchedulePropertyUpdate();
}

void MapObjectPropertyManager::SelectionsRemoved(MapSelectionManager* mgr, const MapSelectionList& sellist) {
        wxASSERT_MSG(mgr==&_editor->GetSelectionManager(),
                        wxT("MapObjectPropertyManager::SelectionsRemoved() called from wrong selection manager"));
        MapSelectionList::Node* node;
        for (node=sellist.GetFirst();node;node=node->GetNext())
                node->GetData()->GetContent()->UnregisterChangeListener(this);
        _selectionChanged=true;
        SchedulePropertyUpdate();
}

