#include "MapProjectionMenuManager.h"

#include "MapEditor.h"
#include "MapProjection.h"
#include "MapProjectionDescriptor.h"

#include <Math/TDRectangle.h>

IMPLEMENT_CLASS(MapProjectionMenuManager,wxEvtHandler);

MapProjectionMenuManager* MapProjectionMenuManager::_instance=NULL;
MapProjectionToIDMap MapProjectionMenuManager::_descriptorToID;
MapProjectionFromIDMap MapProjectionMenuManager::_descriptorFromID;

MapProjectionMenuManager::MapProjectionMenuManager():
        wxEvtHandler()
{
}

MapProjectionMenuManager::~MapProjectionMenuManager() {
}

void MapProjectionMenuManager::UpdateMenu(wxMenu* menu) {
        const MapProjectionDescriptorMap& descs=MapProjectionDescriptor::GetDescriptors();
        MapProjectionFromIDMap::iterator it,delit;
        MapProjectionToIDMap::iterator toit;
        MapProjectionDescriptorMap::const_iterator descit;
        
        // step 1: update the hash and menu to add additional projections
        for (descit=descs.begin();descit!=descs.end();descit++) {
                toit=_descriptorToID.find(descit->first);
                int id;
                if (toit!=_descriptorToID.end()) {
                        // already known
                        id=toit->second;
                } else {
                        // not yet known in the hash
                        id=wxNewId();
                        _descriptorToID[descit->first]=id;
                        _descriptorFromID[id]=descit->first;
                }
                
                if (menu->FindItem(id)) {
                        // is already in the menu, so skip
                        continue;
                }
                
                // not yet in the menu, so add
#       ifdef wxHAS_RADIO_MENU_ITEMS
        menu->AppendRadioItem(id,descit->second->GetDescription());
#       else
        menu->Append(id,descit->second->GetDescription());
#endif
        menu->GetEventHandler()->Connect(id,wxEVT_COMMAND_MENU_SELECTED,
                        wxCommandEventHandler(MapProjectionMenuManager::OnProjectionSelected));
        }
        
        // step 2: update the hash to remove unregistered projections
        it=_descriptorFromID.begin();
        while (it!=_descriptorFromID.end()) {
                descit=descs.find(it->second);
                if (descit!=descs.end()) {
                        // was not removed, only update the text
                        menu->SetLabel(it->first,descit->second->GetDescription());
                        it++;
                } else {
                        // was removed, remove the item from the hash and the menu
                        menu->Remove(it->first);
                        menu->GetEventHandler()->Disconnect(it->first);
                        delit=it;
                        it++;
                        _descriptorToID.erase(delit->second);
                        _descriptorFromID.erase(delit);
                }
        }

#       ifdef wxHAS_RADIO_MENU_ITEMS
        // step 3: point the checkbox to the right item in the menu
        MapEditor* editor=(MapEditor*)menu->GetClientData();
        MapProjection* proj=editor->GetProjection();
        MapProjectionDescriptor* desc=proj->GetDescriptor();
        toit=_descriptorToID.find(desc->GetName());
        if (toit!=_descriptorToID.end()) {
                menu->Check(toit->second,true);
        }
#       endif
}

void MapProjectionMenuManager::OnProjectionSelected(wxCommandEvent& evt) {
        MapProjectionFromIDMap::iterator it=_descriptorFromID.find(evt.GetId());
        wxASSERT(it!=_descriptorFromID.end());
        const MapProjectionDescriptorMap& descs=MapProjectionDescriptor::GetDescriptors();
        MapProjectionDescriptorMap::const_iterator descit=descs.find(it->second);
        wxASSERT(descit!=descs.end());
        
        // determine the menu we're working with
        wxMenu* menu=wxDynamicCast(evt.GetEventObject(),wxMenu);
	if (!menu) {
                wxFrame* frame=wxDynamicCast(evt.GetEventObject(),wxFrame);
                wxASSERT(frame);
                wxMenuBar* menubar=frame->GetMenuBar();
                wxASSERT(menubar);
                wxMenuItem* item=menu->FindItem(evt.GetId());
                wxASSERT(item);
                menu=item->GetMenu();
	}

        MapEditor* editor=(MapEditor*)menu->GetClientData();
        
        MapProjection *newProj=descit->second->CreateInstance();
        TDRectangle geoBounds=editor->GetGeographicBounds();
        newProj->Init(geoBounds.GetCenter());
        
        MapProjection *proj=editor->SetProjection(newProj);
        delete proj;
}
