#ifndef MapSelectionHandle_h
#define MapSelectionHandle_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "MapDraggable.h"
#include <Math/point3d.h>

class MapSelection;

/**
 * \brief A MapSelectionHandle provides a handle for the user to manipulate a
 * selected object.
 *
 * This is used, e.g., in PavementMapObject to allow dragging of the sides of a
 * pavement object to modify width or length.
 */
class MapSelectionHandle: public wxObject,
                public MapDraggable
{
        DECLARE_ABSTRACT_CLASS(MapSelectionHandle)
public:
	MapSelectionHandle(MapSelection* selection):
                _selection(selection) {
        }
        virtual ~MapSelectionHandle() {
        }
        
        virtual void StartDrag(Point3D) {
        }
        
        virtual void Drag(Point3D) {
        }
        
        virtual wxString EndDrag()=0;
        
        /**
         * \brief Provide menu actions for this handle.
         */
        virtual void ProvideActionMenu(wxMenu&) {
        }
        
        MapSelection* GetSelection() const { return _selection; }
protected:
        MapSelection* _selection;
};

#endif

